# Backend endpoints

login = '/api/rest-auth/login/'
dataset = '/api/dataset/'
dataset_upload = '/api/dataset/{}/upload/'
dataset_folder_content = '/api/user-dataset/{}/contents/{}/'
v1_datasets = '/api/v1/ui/datasets/'
v1_dataset_annotation_sets = '/api/v1/ui/datasets/{}/annotation-sets/'
v1_dataset_image_annotations = '/api/v1/ui/datasets/{}/images/{}/annotations/'
v1_dataset_images = '/api/v1/ui/datasets/{}/images/'
v1_export_annotations = '/api/v1/ui/annotation-sets/{}/export/?annotation-format={}'
v1_search = '/api/v1/ui/search/'
